/********************************************************************************
    Copyright (C) 2001  Dirk Farin

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#include <libvideogfx/graphics/lowlevel/img2raw.hh>
#include "output/out_directfb.hh"
#include "config.h"



VideoSink_DirectFB::VideoSink_DirectFB()
  : dfb(NULL), primary(NULL), first(true), pic_available(false)
{
}


VideoSink_DirectFB::~VideoSink_DirectFB()
{
  if (primary) primary->Release(primary);
  if (dfb) dfb->Release(dfb);
}

void VideoSink_DirectFB::ShowMBRows(DecodedImageData* dimg)
{
  const Image_YUV<Pixel>& img = dimg->m_image;

  nextpts = dimg->m_timing.pts;

  int sx,sy;

  if (first)
    {
      ImageParam_YUV param;
      img.GetParam(param);
      d_height = param.height;

      StartAccumulation(0,d_height-1,false);

      static int argc=1;
      static char* argvarray[] = { "dvdview", NULL };
      static char** argv = argvarray;

      DFBResult err;
      DFBSurfaceDrawingFlags flags;

      if (DirectFBCreate( &argc,&argv, &dfb )  !=  DFB_OK)
	throw "DirectFB-Error 1";

      dfb->GetSupportedDrawingFlags( dfb, &flags );

      err = dfb->SetCooperativeLevel( dfb, DFSCL_FULLSCREEN );
      if (err != DFB_OK) {
	DirectFBError( "Failed requesting exclusive access", err );
	dfb->Release( dfb );
	throw "DirectFB-Error 2";
      }

#if 0
      err = dfb->SetVideoMode( dfb, sx, sy, 16 );
      if (err != DFB_OK) {
	DirectFBError( "Failed setting video mode", err );
	keybuffer->Release( keybuffer );
	keyboard->Release( keyboard );
	dfb->Release( dfb );
	return 1;
      }
#endif

      DFBSurfaceDescription dsc;

      memset( &dsc, 0, sizeof(DFBSurfaceDescription) );

      dsc.flags = DSDESC_CAPS;
      dsc.caps = (flags & DSDRAW_BLEND) ?
	DSCAPS_PRIMARY | DSCAPS_FLIPPING :
	DSCAPS_PRIMARY | DSCAPS_FLIPPING | DSCAPS_SYSTEMONLY;

      err = dfb->CreateSurface( dfb, &dsc, &primary );
      if (err != DFB_OK) {
	DirectFBError( "Failed creating primary surface", err );
	dfb->Release( dfb );
	throw "DirectFB-Error 4";
      }

      primary->GetSize( primary, &sx, &sy );

      primary->SetColor( primary, 0x00, 0x00, 0x00, 0x00 );
      primary->FillRectangle( primary, 0, 0, sx, sy );
      err = primary->Flip( primary, NULL, 0 );
      if (err != DFB_OK) {
	DirectFBError( "Failed flipping the primary surface", err );
	primary->Release( primary );
	dfb->Release( dfb );
	throw "DirectFB-Error 5";
      }

      first=false;
    }

#if 1
  DecodedImageData* decimg = Accumulate(dimg);
  if (!decimg)
    return;
#endif

#if 1
  int pitch;
  void* data;

  primary->Lock(primary,DSLF_WRITE,&data,&pitch);

  RawImageSpec_RGB spec;
  spec.bytes_per_line = pitch;
  spec.bits_per_pixel = 16;
  spec.little_endian  = true;
  spec.SetRGBMasks(0xF800,0x07E0,0x001F);

  Image2Raw transformation;
  transformation.SetOutputSpec(spec);
  transformation.TransformYUV(decimg->m_image , (uint8*)(data));

  if (primary->Unlock( primary ) == DFB_OK)
    primary->Flip( primary, NULL, DSFLIP_BLIT );
#endif
}


void VideoSink_DirectFB::FinishedPicture()
{
  pic_available=true;
  StartAccumulation(0,d_height-1,false);
}

